from rest_framework import serializers
from .models import ElectionVote, UniqueVote


class AnonymizeVoteSerializer(serializers.Serializer):
    election_id = serializers.CharField(required=True,
                                        help_text="Election ID " +
                                                  "signed by" +
                                                  " the Voting System")
    cyphered_vote = serializers.CharField(required=True,
                                          allow_blank=False,
                                          help_text="Cyphered vote")
    signed_vote = serializers.CharField(required=True,
                                        allow_blank=False,
                                        help_text="Cyphered vote hash" +
                                                  " signed by" +
                                                  " the Validator Service")
    blind_components = serializers.CharField(required=True,
                                             allow_blank=False,
                                             help_text="Blind signature" +
                                                       " components")
    pr_components = serializers.CharField(required=True,
                                          allow_blank=False,
                                          help_text="Blind PR signature" +
                                                    " components")
    unique_bulletin = serializers.CharField(required=True,
                                            allow_blank=False,
                                            help_text="Unique bulletin number")

    def validate(self, data):
        """
        Complete object validation.
        Access fields using array dictionary notation data['field']
        Errors should raise a ValidationError Exception
        """
        data = super(AnonymizeVoteSerializer, self).validate(data)

        return data

    def validate_election_id(self, value):
        """ Check if Election is properly initialized """
        entry = ElectionVote.objects.filter(election_id=value)[:1]
        if not entry.exists():
            raise serializers.ValidationError("Election not initialized")

        return value


class InitializeElectionSerializer(serializers.Serializer):
    voters_nr = serializers.IntegerField(required=True,
                                         help_text="Number of voters " +
                                                   "assigned to the " +
                                                   "given election ")

    def validate(self, data):
        """
        Complete object validation.
        Access fields using array dictionary notation data['field']
        Errors should raise a ValidationError Exception
        """
        data = super(InitializeElectionSerializer, self).validate(data)

        return data


class CloseElectionSerializer(serializers.Serializer):

    def validate(self, data):
        """
        Complete object validation.
        Access fields using array dictionary notation data['field']
        Errors should raise a ValidationError Exception
        """
        data = super(CloseElectionSerializer, self).validate(data)

        return data


class UniqueVoteSerializer(serializers.ModelSerializer):
    class Meta:
        model = UniqueVote
        fields = ('unique_vote_bulletin', )


class ElectionVoteSerializer(serializers.ModelSerializer):
    class Meta:
        model = ElectionVote
        fields = ('id', 'election_id', 'cyphered_vote',
                  'blind_components', 'pr_components',
                  'signed_vote_hash')


class UpdateVotesSerializer(serializers.Serializer):
    vote_list = serializers.ListField(
                    child=serializers.DictField(
                        child=serializers.CharField()))

    def validate(self, data):
        """
        Complete object validation.
        Access fields using array dictionary notation data['field']
        Errors should raise a ValidationError Exception
        """
        data = super(UpdateVotesSerializer, self).validate(data)

        return data
